require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xdbpackage',
  dscmeta_resource_friendly_name: 'xDBPackage',
  dscmeta_resource_name: 'MSFT_xDBPackage',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xDatabase',
  dscmeta_module_version: '2.0.0',
  docs: 'The DSC xDBPackage resource type.
         Automatically generated from version 2.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_sqlserver: {
      type: 'String',
      desc: 'Sql Server Name',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlserverversion: {
      type: "Enum['2008-R2', '2012', '2014', '2016', '2017', '2019']",
      desc: 'Sql Server Version For DacFx',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_type: {
      type: "Enum['DACPAC', 'BACPAC']",
      desc: 'Type for backup(Extract id done for DACPAC and Import for BACPAC)',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_databasename: {
      type: 'String',
      desc: 'Name of the Database',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_path: {
      type: 'String',
      desc: 'Path to BacPac/DacPac',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credentials: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials to Connect to the sql server',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
  },
)
